"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildSwaggerPathsFromControllers = exports.buildRoutesFromController = void 0;
require("reflect-metadata");
const decorators_1 = require("./decorators");
const joi_to_swagger_1 = __importDefault(require("joi-to-swagger"));
const lodash_1 = require("lodash");
function buildRoutesFromController(controller) {
    const controllerProps = Reflect.getMetadata(decorators_1.controllerMetaKey, controller);
    const methodMappings = Reflect.getMetadata(decorators_1.methodsMetaKey, controller);
    if (!controllerProps || !methodMappings)
        throw new Error('Invalid controller provided');
    const rawBasePath = controllerProps.basePath;
    if (!rawBasePath)
        throw new Error('Provided object is not a valid controller');
    const basePath = rawBasePath.replace(/\/$/, '');
    const routes = {};
    for (const { method, path: rawResourcePath, propertyKey, logRequest, logResponse } of Object.values(methodMappings)) {
        const resourcePath = rawResourcePath ? rawResourcePath.replace(/(^)([^/])/, '/$2').replace(/\/$/, '') : '';
        const path = basePath + resourcePath;
        routes[path] = routes[path] || { path };
        switch (method) {
            case 'GET':
                routes[path].get = {
                    handler: controller[propertyKey],
                    matchedPath: path,
                    matchedMethod: method,
                    tracingConfig: {
                        logRequest: Boolean(logRequest),
                        logResponse: Boolean(logResponse),
                    },
                };
                break;
            case 'POST':
                routes[path].post = {
                    handler: controller[propertyKey],
                    matchedPath: path,
                    matchedMethod: method,
                    tracingConfig: {
                        logRequest: Boolean(logRequest),
                        logResponse: Boolean(logResponse),
                    },
                };
                break;
            case 'PUT':
                routes[path].put = {
                    handler: controller[propertyKey],
                    matchedPath: path,
                    matchedMethod: method,
                    tracingConfig: {
                        logRequest: Boolean(logRequest),
                        logResponse: Boolean(logResponse),
                    },
                };
                break;
            case 'PATCH':
                routes[path].patch = {
                    handler: controller[propertyKey],
                    matchedPath: path,
                    matchedMethod: method,
                    tracingConfig: {
                        logRequest: Boolean(logRequest),
                        logResponse: Boolean(logResponse),
                    },
                };
                break;
            case 'DELETE':
                routes[path].delete = {
                    handler: controller[propertyKey],
                    matchedPath: path,
                    matchedMethod: method,
                    tracingConfig: {
                        logRequest: Boolean(logRequest),
                        logResponse: Boolean(logResponse),
                    },
                };
                break;
        }
    }
    return Object.values(routes);
}
exports.buildRoutesFromController = buildRoutesFromController;
function buildSwaggerPathsFromControllers(controllers) {
    const paths = {};
    for (const controller of controllers) {
        const controllerProps = Reflect.getMetadata(decorators_1.controllerMetaKey, controller);
        const methodMappings = Reflect.getMetadata(decorators_1.methodsMetaKey, controller);
        if (!controllerProps || !methodMappings)
            throw new Error('Invalid controller provided');
        const rawBasePath = controllerProps.basePath;
        const tags = controllerProps.tags;
        if (!rawBasePath)
            throw new Error('Provided object is not a valid controller');
        const basePath = rawBasePath.replace(/\/$/, '');
        for (const { method, path: rawResourcePath, parameters, responseExamples, description } of Object.values(methodMappings)) {
            const resourcePath = rawResourcePath ? rawResourcePath.replace(/(^)([^/])/, '/$2').replace(/\/$/, '') : '';
            const path = basePath + resourcePath;
            const lowerCaseMethod = method.toLowerCase();
            paths[path] = paths[path] || {};
            paths[path][lowerCaseMethod] = {
                tags,
            };
            paths[path][lowerCaseMethod]['parameters'] = [];
            if (parameters) {
                const { pathParametersSchema, queryStringParametersSchema, headersSchema, bodySchema } = parameters;
                if (pathParametersSchema) {
                    const param = getParameters(pathParametersSchema, 'path');
                    paths[path][lowerCaseMethod]['parameters'].push(...param);
                }
                if (queryStringParametersSchema) {
                    const param = getParameters(queryStringParametersSchema, 'query');
                    paths[path][lowerCaseMethod]['parameters'].push(...param);
                }
                if (headersSchema) {
                    const param = getParameters(headersSchema, 'header');
                    paths[path][lowerCaseMethod]['parameters'].push(...param);
                }
                if (bodySchema) {
                    const { schema, examples } = getBodySchema(bodySchema);
                    paths[path][lowerCaseMethod]['requestBody'] = {
                        content: {
                            'application/json': {
                                schema,
                                examples,
                            },
                        },
                    };
                }
            }
            if (responseExamples) {
                for (const [code, samples] of Object.entries(responseExamples)) {
                    paths[path][lowerCaseMethod]['responses'] =
                        {
                            ...paths[path][lowerCaseMethod]['responses'],
                            [String(code)]: {
                                content: {
                                    'application/json': {
                                        examples: {},
                                    },
                                },
                            },
                        } || paths[path][lowerCaseMethod]['responses'];
                    for (const sample of samples) {
                        const index = Object.keys(paths[path][lowerCaseMethod]['responses'][String(code)]['content']['application/json']['examples']).length;
                        paths[path][lowerCaseMethod]['responses'][String(code)]['content']['application/json']['examples'][`Example${index}`] = { value: sample };
                    }
                }
            }
            if (description) {
                paths[path][lowerCaseMethod]['description'] = description;
            }
        }
    }
    return paths;
}
exports.buildSwaggerPathsFromControllers = buildSwaggerPathsFromControllers;
function getParameters(schema, location) {
    const param = [];
    const { swagger } = joi_to_swagger_1.default(schema);
    const required = swagger.required || [];
    if (swagger && swagger.properties) {
        for (const [name, schema] of Object.entries(swagger.properties)) {
            const { examples: rawExamples, example: rawExample } = schema;
            const examples = (rawExamples ? rawExamples : rawExample ? [rawExample] : []).reduce((examples, example, idx) => {
                examples[`Example${idx}`] = { value: example };
                return examples;
            }, {});
            const newSchema = lodash_1.omit(schema, ['required', 'description', 'examples', 'example']);
            const { description } = schema;
            param.push({
                in: location,
                name,
                schema: newSchema,
                required: required.includes(name),
                description,
                examples,
            });
        }
    }
    return param;
}
function getBodySchema(schema) {
    const { swagger } = joi_to_swagger_1.default(schema);
    const { examples: rawExamples, example: rawExample } = swagger;
    const examples = lodash_1.cloneDeep((rawExamples ? rawExamples : rawExample ? [rawExample] : []).reduce((examples, example, idx) => {
        examples[`Example${idx}`] = { value: example };
        return examples;
    }, {}));
    delete swagger.examples;
    delete swagger.example;
    return { schema: swagger, examples };
}
